"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serialiseRequestV2 = exports.serialiseRequest = void 0;
const pad = (hexNum) => "0".repeat(6 - hexNum.length) + hexNum;
const prependLen = (message) => pad(message.length.toString(16)) + message;
const serialiseArgs = (req) => {
    switch (req.type) {
        case ":add-clause":
        case ":add-missing":
        case ":case-split":
        case ":generate-def":
        case ":make-case":
        case ":make-lemma":
        case ":make-with":
            return `${req.type} ${req.line} "${req.name}"`;
        case ":apropos":
            return `${req.type} "${req.string}"`;
        case ":browse-namespace":
            return `${req.type} "${req.namespace}"`;
        case ":calls-who":
        case ":print-definition":
        case ":repl-completions":
        case ":type-of":
        case ":who-calls":
            return `${req.type} "${req.name}"`;
        case ":docs-for":
            return `${req.type} "${req.name}" ${req.mode}`;
        case ":interpret":
            return `${req.type} "${req.expression}"`;
        case ":load-file":
            return `${req.type} "${req.path}"`;
        case ":metavariables":
            return `${req.type} ${req.width}`;
        case ":proof-search":
            return `${req.type} ${req.line} "${req.name}" (${req.hints.join(" ")})`;
        case ":type-at":
            return `:type-of "${req.name}" ${req.line} ${req.column}`;
        case ":generate-def-next":
        case ":proof-search-next":
        case ":version":
            return req.type;
    }
};
/**
 * Serialise request according to the Idris 1 IDE protocol. Arguments, including
 * the request type are always an s-expression list.
 */
const serialiseRequest = (req) => prependLen(`((${serialiseArgs(req)}) ${req.id})\n`);
exports.serialiseRequest = serialiseRequest;
/**
 * The Idris 2 IDE protocol introduces some new commands, and at least one
 * breaking change to the serialised request format. Specifically commands
 * without arguments are now simply atoms, rather than single-element lists.
 */
const serialiseRequestV2 = (req) => {
    if (req.type === ":generate-def-next" ||
        req.type === ":proof-search-next" ||
        req.type === ":version") {
        return prependLen(`(${serialiseArgs(req)} ${req.id})\n`);
    }
    else {
        return prependLen(`((${serialiseArgs(req)}) ${req.id})\n`);
    }
};
exports.serialiseRequestV2 = serialiseRequestV2;
//# sourceMappingURL=request.js.map