"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleWarning = void 0;
const path_1 = require("path");
const vscode = require("vscode");
const state_1 = require("../state");
const warningToDiagnostic = (reply) => new vscode.Diagnostic(diagnosticRange(reply), rmLocDesc(reply.err.warning));
const diagnosticRange = (reply) => {
    const { start, end, filename } = reply.err;
    const isLidr = path_1.extname(filename) === ".lidr";
    const { idris2Mode } = state_1.state;
    // In Idris 2 0.6.0, the positions returned by the IDE process were updated to
    // be 0-indexed like VS. Also in Idris2, .lidr positions are returned as if
    // they’re normal Idris files, so they’re off by two, for the `> ` at the
    // beginning. In Idris 1, .lidr files are correctly indexed, but all
    // positions are 1-indexed.
    return idris2Mode
        ? isLidr
            ? new vscode.Range(new vscode.Position(start.line, start.column + 2), new vscode.Position(end.line, end.column + 2))
            : new vscode.Range(new vscode.Position(start.line, start.column), new vscode.Position(end.line, end.column))
        : new vscode.Range(new vscode.Position(start.line - 1, start.column - 1), new vscode.Position(end.line - 1, end.column));
};
const filenameRegex = /.*:\d+:\d+--\d+:\d+\n(.|\n)*?(\n\n|\n$)/;
const parseLineRegex = /Parse error at line \d+:\d+:\n/;
const rmFileSec = (warning) => warning.replace(filenameRegex, "");
const rmParseLine = (warning) => warning.replace(parseLineRegex, "");
const rmLocDesc = (warning) => rmFileSec(rmParseLine(warning)).trim();
const handleWarning = (reply) => {
    const { diagnostics, idrisProcDir } = state_1.state;
    const filename = reply.err.filename;
    // Idris2 sometimes uses relative file paths, which aren’t parsed into file URIs correctly on their own.
    const uri = path_1.isAbsolute(filename) ? vscode.Uri.file(filename) : vscode.Uri.file(path_1.join(idrisProcDir || "", filename));
    const existing = diagnostics.get(uri) || [];
    diagnostics.set(uri, existing.concat(warningToDiagnostic(reply)));
};
exports.handleWarning = handleWarning;
//# sourceMappingURL=diagnostics.js.map